/*
cpphtml
a C++ to HTML converter by Bob Rost
converts C++ code to NetPositive-friendly HTML with BeIDE color scheme
*/

#include <stdio.h>
#define bool unsigned char
#define true 1
#define false 0

bool comments=false;
bool strings=false;
bool chars=false;

bool isletter(char c) /*is a char a letter or number?*/
	{
	return (('a'<=c&&c<='z')||('A'<=c&&c<='Z')||('0'<=c&&c<='9'));
	}

void printout(char c) /*prints an html-friendly char*/
	{
	switch(c)
		{
		case '<': printf("&lt;"); break;
		case '>': printf("&gt;"); break;
		case '&': printf("&amp;"); break;
		default: printf("%c",c);
		}
	}

int printword(const char *s)
	{
	int i=0;
	if (!isletter(s[0]))
		{
		printout(s[0]);
		return 1;
		}
	while (isletter(s[i])) printout(s[i++]);
	return i;
	}

bool match(const char *s, const char *str)
	{
	int i=0;
	while (str[i]!=0) if(s[i]!=str[i++]) return false;
	if (isletter(s[i])) return false;
	return true;
	}

bool match2(const char *s, const char *str) /*doesn't check if something is a letter*/
	{
	int i=0;
	while (str[i]!=0) if(s[i]!=str[i++]) return false;
	return true;
	}

void handle(const char *s)
	{
	int i=0;
	if (s[0]==0) return;
	if(comments) /*continue commented code*/
	  {
	  while(!(match2(s+i,"*/")||s[i]==0)) printout(s[i++]);
	  if (s[i]!=0)
	    {
	    comments=false;
	    printf("*/</font>");
	    handle(s+i+2);
	    }
	  }
	else if(chars) /*continue char text*/
	  {
	  if (match2(s,"\\\""))
	    {
	    printf("\\\"");
	    handle(s+2);
	    }
	  else if (match2(s,"\\\'"))
	    {
	    printf("\\\'");
	    handle(s+2);
	    }
	  else if (s[0]=='\'')
	    {
	    printf("\'</font>");
	    chars=false;
	    handle(s+1);
	    }
	  else
	    {
	    printout(s[0]);
	    handle(s+1);
	    }
	  }
	else if(strings) /*continue string text*/
	  {
	  if (match2(s,"\\\""))
	    {
	    printf("\\\"");
	    handle(s+2);
	    }
	  else if (match2(s,"\\\'"))
	    {
	    printf("\\\'");
	    handle(s+2);
	    }
	  else if (s[0]=='\"')
	    {
	    printf("\"</font>");
	    strings=false;
	    handle(s+1);
	    }
	  else
	    {
	    printout(s[0]);
	    handle(s+1);
	    }
	  }
	else if (match2(s,"/*")) /*block comment start*/
	  {
	  printf("<font color=\"red\">/*");
	  comments=true; strings=chars=false;
	  handle(s+2);
	  }
	else if (match2(s,"//")) /*line comment handler*/
	  {
	  strings=chars=false;
	  printf("<font color=\"red\">");
	  while (s[i]!=0) printout(s[i++]);
	  printf("</font>");
	  }
	else if (s[0]=='\"') /*start string text*/
	  {
	  printf("<font color=\"#888888\">\"");
	  strings=true;
	  handle(s+1);
	  }
	else if (s[0]=='\'') /*start char text*/
	  {
	  printf("<font color=\"#888888\">\'");
	  chars=true;
	  handle(s+1);
	  }
	/*keyword handlers*/
	else if (match(s,"int")){printf("<font color=\"blue\">int</font>");handle(s+3);}
	else if (match(s,"return")){printf("<font color=\"blue\">return</font>");handle(s+6);}
	else if (match(s,"float")){printf("<font color=\"blue\">float</font>");handle(s+5);}
	else if (match(s,"double")){printf("<font color=\"blue\">double</font>");handle(s+6);}
	else if (match(s,"#include")){printf("<font color=\"blue\">#include</font>");handle(s+8);}
	else if (match(s,"new")){printf("<font color=\"blue\">new</font>");handle(s+3);}
	else if (match(s,"if")){printf("<font color=\"blue\">if</font>");handle(s+2);}
	else if (match(s,"else")){printf("<font color=\"blue\">else</font>");handle(s+4);}
	else if (match(s,"char")){printf("<font color=\"blue\">char</font>");handle(s+4);}
	else if (match(s,"delete")){printf("<font color=\"blue\">delete</font>");handle(s+6);}
	else if (match(s,"bool")){printf("<font color=\"blue\">bool</font>");handle(s+4);}
	else if (match(s,"unsigned")){printf("<font color=\"blue\">unsigned</font>");handle(s+8);}
	else if (match(s,"void")){printf("<font color=\"blue\">void</font>");handle(s+4);}
	else if (match(s,"while")){printf("<font color=\"blue\">while</font>");handle(s+5);}
	else if (match(s,"class")){printf("<font color=\"blue\">class</font>");handle(s+5);}
	else if (match(s,"public")){printf("<font color=\"blue\">public</font>");handle(s+6);}
	else if (match(s,"true")){printf("<font color=\"blue\">true</font>");handle(s+4);}
	else if (match(s,"false")){printf("<font color=\"blue\">false</font>");handle(s+5);}
	else if (match(s,"switch")){printf("<font color=\"blue\">switch</font>");handle(s+6);}
	else if (match(s,"default")){printf("<font color=\"blue\">default</font>");handle(s+7);}
	else if (match(s,"case")){printf("<font color=\"blue\">case</font>");handle(s+4);}
	else if (match(s,"const")){printf("<font color=\"blue\">const</font>");handle(s+5);}
	else if (match(s,"#define")){printf("<font color=\"blue\">#define</font>");handle(s+7);}
	else if (match(s,"#ifndef")){printf("<font color=\"blue\">#ifndef</font>");handle(s+7);}
	else if (match(s,"#endif")){printf("<font color=\"blue\">#endif</font>");handle(s+6);}
	else if (match(s,"try")){printf("<font color=\"blue\">try</font>");handle(s+3);}
	else if (match(s,"catch")){printf("<font color=\"blue\">catch</font>");handle(s+5);}
	else if (match(s,"private")){printf("<font color=\"blue\">private</font>");handle(s+7);}
	else if (match(s,"protected")){printf("<font color=\"blue\">protected</font>");handle(s+9);}
	else if (match(s,"namespace")){printf("<font color=\"blue\">namespace</font>");handle(s+9);}
	else if (match(s,"inline")){printf("<font color=\"blue\">inline</font>");handle(s+6);}
	else if (match(s,"virtual")){printf("<font color=\"blue\">virtual</font>");handle(s+7);}
	else if (match(s,"explicit")){printf("<font color=\"blue\">explicit</font>");handle(s+8);}
	else if (match(s,"template")){printf("<font color=\"blue\">template</font>");handle(s+8);}
	else if (match(s,"typename")){printf("<font color=\"blue\">typename</font>");handle(s+8);}
	else if (match(s,"typedef")){printf("<font color=\"blue\">typedef</font>");handle(s+7);}
	else if (match(s,"operator")){printf("<font color=\"blue\">operator</font>");handle(s+8);}
	else {handle(s+printword(s));} /*catch substring keywords*/
	}


int main(int argn, char **argv)
{
char buff[1024];
FILE *fp;
if (argn!=2)
  {
  printf("usage: %s [input cpp file]\n",argv[0]);
  return 1;
  }
fp=fopen(argv[1],"r");
printf("<html>\n<pre>\n");
while (fgets(buff,1024,fp))
{
handle(buff);
}
printf("</pre>\n</html>\n");
}
